/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

public class ViewFrame extends JFrame implements WindowListener {

    private static final long serialVersionUID = -7037635097739548470L;

    private final ViewPanel viewPanel;

    public ViewFrame() {
        setTitle("3D engine");

        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(final java.awt.event.WindowEvent e) {
                exit();
            }
        });

        viewPanel = new ViewPanel();

        add(getViewPanel());

        setSize(800, 600);

        // maximize window
        setExtendedState(JFrame.MAXIMIZED_BOTH);
        setVisible(true);

        addResizeListener();
        addWindowListener(this);
    }

    private void addResizeListener() {
        addComponentListener(new ComponentListener() {
            // This method is called after the component's size changes
            @Override
            public void componentHidden(final ComponentEvent e) {
            }

            @Override
            public void componentMoved(final ComponentEvent e) {
            }

            @Override
            public void componentResized(final ComponentEvent evt) {

                final Component c = (Component) evt.getSource();

                // Get new size
                final Dimension newSize = c.getSize();

                boolean sizeFixed = false;

                if (newSize.width < 400) {
                    newSize.width = 400;
                    sizeFixed = true;
                }

                if (newSize.height < 400) {
                    newSize.height = 400;
                    sizeFixed = true;
                }

                if (sizeFixed)
                    setSize(newSize);

            }

            @Override
            public void componentShown(final ComponentEvent e) {
                viewPanel.repaintDuringNextViewUpdate();
            }

        });
    }

    public void exit() {
        if (getViewPanel() != null) {
            getViewPanel().stop();
            getViewPanel().setEnabled(false);
            getViewPanel().setVisible(false);
        }
        dispose();
    }

    @Override
    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(640, 480);
    }

    /**
     * @return the view
     */
    public ViewPanel getViewPanel() {
        return viewPanel;
    }

    @Override
    public void windowActivated(final WindowEvent e) {
        viewPanel.repaintDuringNextViewUpdate();
    }

    @Override
    public void windowClosed(final WindowEvent e) {
    }

    @Override
    public void windowClosing(final WindowEvent e) {
    }

    @Override
    public void windowDeactivated(final WindowEvent e) {
    }

    @Override
    public void windowDeiconified(final WindowEvent e) {
        viewPanel.repaintDuringNextViewUpdate();
    }

    @Override
    public void windowIconified(final WindowEvent e) {
    }

    @Override
    public void windowOpened(final WindowEvent e) {
        viewPanel.repaintDuringNextViewUpdate();
    }

}
