/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.math;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;

/**
 * Used to represent transformation in a 3D space.
 */
public class Orientation implements Cloneable {


    private double s1, c1, s2, c2;

    /**
     * The angle of rotation around the XZ axis.
     */
    private double angleXZ = 0;

    /**
     * The angle of rotation around the YZ axis.
     */
    private double angleYZ = 0;

    public Orientation() {
        computeMultipliers();
    }

    public Orientation(final double angleXZ, final double angleYZ) {
        this.angleXZ = angleXZ;
        this.angleYZ = angleYZ;
        computeMultipliers();
    }

    @Override
    public Orientation clone() {
        return new Orientation(angleXZ, angleYZ);
    }

    /**
     * Computes the sine and cosine of the angles.
     */
    private void computeMultipliers() {
        s1 = Math.sin(angleXZ);
        c1 = Math.cos(angleXZ);

        s2 = Math.sin(angleYZ);
        c2 = Math.cos(angleYZ);
    }

    public void rotate(final Point3D point3d) {
        final double z1 = (point3d.z * c1) - (point3d.x * s1);
        point3d.x = (point3d.z * s1) + (point3d.x * c1);

        point3d.z = (z1 * c2) - (point3d.y * s2);
        point3d.y = (z1 * s2) + (point3d.y * c2);
    }

    public void rotate(final double angleXZ, final double angleYZ) {
        this.angleXZ += angleXZ;
        this.angleYZ += angleYZ;
        computeMultipliers();
    }

}
