/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.octree.raytracer;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.Avatar;

import static eu.svjatoslav.sixth.e3d.renderer.octree.raytracer.Camera.SIZE;

/**
 * Represents camera view. Used to compute direction of rays during ray tracing.
 */
public class CameraView {

    /**
     * Camera view coordinates.
     */
    Point3D cameraCenter, topLeft, topRight, bottomLeft, bottomRight;

    public CameraView(final Avatar avatar, final double zoom) {
        // compute camera view coordinates as if camera is at (0,0,0) and look at (0,0,1)
        final float viewAngle = (float) .6;
        cameraCenter = new Point3D();
        topLeft = new Point3D(0, 0, SIZE).rotate(-viewAngle, -viewAngle);
        topRight = new Point3D(0, 0, SIZE).rotate(viewAngle, -viewAngle);
        bottomLeft = new Point3D(0, 0, SIZE).rotate(-viewAngle, viewAngle);
        bottomRight = new Point3D(0, 0, SIZE).rotate(viewAngle, viewAngle);

        topLeft.rotate(-avatar.getAngleXZ(), -avatar.getAngleYZ());
        topRight.rotate(-avatar.getAngleXZ(), -avatar.getAngleYZ());
        bottomLeft.rotate(-avatar.getAngleXZ(), -avatar.getAngleYZ());
        bottomRight.rotate(-avatar.getAngleXZ(), -avatar.getAngleYZ());

        // place camera view at avatar location
        avatar.getLocation().clone().scaleDown(zoom).addTo(cameraCenter, topLeft, topRight, bottomLeft, bottomRight);
    }

}
