/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster;

import eu.svjatoslav.sixth.e3d.gui.RenderingContext;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.AbstractCoordinateShape;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Comparator;

public class RenderAggregator {

    private final ArrayList<AbstractCoordinateShape> shapes = new ArrayList<>();
    private final ShapesZIndexComparator comparator = new ShapesZIndexComparator();

    public void paint(final RenderingContext renderBuffer) {
        shapes.sort(comparator);
        shapes.forEach(shape -> shape.paint(renderBuffer));
    }

    public void queueShapeForRendering(final AbstractCoordinateShape shape) {
        shapes.add(shape);
    }

    public void reset() {
        shapes.clear();
    }

    static class ShapesZIndexComparator implements Comparator<AbstractCoordinateShape>, Serializable {

        @Override
        public int compare(final AbstractCoordinateShape o1, final AbstractCoordinateShape o2) {
            if (o1.getZ() < o2.getZ())
                return 1;
            else if (o1.getZ() > o2.getZ())
                return -1;

            return Integer.compare(o1.shapeId, o2.shapeId);
        }
    }
}
