/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.textcanvas;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.RenderingContext;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.AbstractCoordinateShape;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.solidpolygon.SolidPolygon;

import java.awt.*;

public class CanvasCharacter extends AbstractCoordinateShape {

    private static final int MAX_FONT_SIZE = 500;
    private static final Font[] fonts = new Font[MAX_FONT_SIZE];
    private String value;

    private eu.svjatoslav.sixth.e3d.renderer.raster.Color foregroundColor;
    private Color foregroundAwtColor;

    private eu.svjatoslav.sixth.e3d.renderer.raster.Color backgroundColor;

    public CanvasCharacter(final Point3D point, final char character,
                           final eu.svjatoslav.sixth.e3d.renderer.raster.Color foregroundColor,
                           final eu.svjatoslav.sixth.e3d.renderer.raster.Color backgroundColor) {

        super(5);
        coordinates[0].coordinate = point;

        value = String.valueOf(character);

        this.foregroundColor = foregroundColor;
        foregroundAwtColor = foregroundColor.toAwtColor();

        this.backgroundColor = backgroundColor;

        // set corner coordinates (for drawing background)
        {
            final double widthHalf = TextCanvas.FONT_CHAR_WIDTH / 2d;
            final double heightHalf = TextCanvas.FONT_CHAR_HEIGHT / 2d;

            // upper left
            coordinates[1].coordinate = point.clone().translateX(-widthHalf)
                    .translateY(-heightHalf);

            // upper right
            coordinates[2].coordinate = point.clone().translateX(widthHalf)
                    .translateY(-heightHalf);

            // lower right
            coordinates[3].coordinate = point.clone().translateX(widthHalf)
                    .translateY(heightHalf);

            // lower left
            coordinates[4].coordinate = point.clone().translateX(-widthHalf)
                    .translateY(heightHalf);

        }
    }

    public static Font getFont(final int size) {
        if (fonts[size] != null)
            return fonts[size];

        final Font font = new Font("Courier", Font.BOLD, size);
        fonts[size] = font;
        return font;
    }

    public eu.svjatoslav.sixth.e3d.renderer.raster.Color getBackgroundColor() {
        return backgroundColor;
    }

    public void setBackgroundColor(
            final eu.svjatoslav.sixth.e3d.renderer.raster.Color backgroundColor) {
        this.backgroundColor = backgroundColor;
    }

    public eu.svjatoslav.sixth.e3d.renderer.raster.Color getForegroundColor() {
        return foregroundColor;
    }

    public void setForegroundColor(
            final eu.svjatoslav.sixth.e3d.renderer.raster.Color foregroundColor) {
        this.foregroundColor = foregroundColor;
        foregroundAwtColor = foregroundColor.toAwtColor();
    }

    @Override
    public void paint(final RenderingContext renderingContext) {

        SolidPolygon.drawPolygon(renderingContext,
                coordinates[1].onScreenCoordinate,
                coordinates[2].onScreenCoordinate,
                coordinates[3].onScreenCoordinate, mouseInteractionController,
                backgroundColor);

        SolidPolygon.drawPolygon(renderingContext,
                coordinates[1].onScreenCoordinate,
                coordinates[3].onScreenCoordinate,
                coordinates[4].onScreenCoordinate, mouseInteractionController,
                backgroundColor);

        final int size = (int) ((renderingContext.width * 4.5) / onScreenZ);

        // do not render too large characters
        if (size >= MAX_FONT_SIZE)
            return;

        final Point2D onScreenLocation = coordinates[0].onScreenCoordinate;

        // screen borders check
        if (onScreenLocation.x < 0)
            return;
        if (onScreenLocation.y < 0)
            return;

        if (onScreenLocation.x > renderingContext.width)
            return;
        if (onScreenLocation.y > renderingContext.height)
            return;

        renderingContext.graphics.setFont(getFont(size));
        renderingContext.graphics.setColor(foregroundAwtColor);
        renderingContext.graphics.drawString(
                value,
                (int) onScreenLocation.x - (int) (size / 3.2),
                (int) onScreenLocation.y + (int) (size / 2.5));

    }

    public void setValue(final char value) {
        this.value = String.valueOf(value);
    }

}
