/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.slicer;

import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.texturedpolygon.TexturedPolygon;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class Slicer {

    private final double maxDistance;
    private final List<TexturedPolygon> result = new ArrayList<>();

    public Slicer(final double maxDistance) {
        this.maxDistance = maxDistance;
    }

    private void considerSlicing(final PolygonCoordinate c1,
                                 final PolygonCoordinate c2, final PolygonCoordinate c3,
                                 final TexturedPolygon originalPolygon) {

        final BorderLine[] lines = new BorderLine[]{
                new BorderLine(c1, c2, 1), new BorderLine(c2, c3, 2),
                new BorderLine(c3, c1, 3)};

        Arrays.sort(lines, lines[0]);

        final BorderLine longestLine = lines[2];

        if (longestLine.getLength() < maxDistance) {
            final TexturedPolygon polygon = new TexturedPolygon(c1, c2, c3,
                    originalPolygon.texture);

            polygon.setMouseInteractionController(originalPolygon.mouseInteractionController);

            getResult().add(polygon);
            return;
        }

        final PolygonCoordinate middle = longestLine.getMiddlePoint();

        switch (longestLine.count) {
            case 1:
                considerSlicing(c1, middle, c3, originalPolygon);
                considerSlicing(middle, c2, c3, originalPolygon);
                return;
            case 2:
                considerSlicing(c1, c2, middle, originalPolygon);
                considerSlicing(middle, c3, c1, originalPolygon);
                return;
            case 3:
                considerSlicing(c1, c2, middle, originalPolygon);
                considerSlicing(middle, c2, c3, originalPolygon);
        }

    }

    public List<TexturedPolygon> getResult() {
        return result;
    }

    public void slice(final TexturedPolygon originalPolygon) {

        final PolygonCoordinate pc1 = new PolygonCoordinate(
                originalPolygon.coordinates[0].coordinate,
                originalPolygon.texturePoint1);

        final PolygonCoordinate pc2 = new PolygonCoordinate(
                originalPolygon.coordinates[1].coordinate,
                originalPolygon.texturePoint2);

        final PolygonCoordinate pc3 = new PolygonCoordinate(
                originalPolygon.coordinates[2].coordinate,
                originalPolygon.texturePoint3);

        considerSlicing(pc1, pc2, pc3, originalPolygon);
    }

}
